package de.bioinf.base;

import java.util.Arrays;
import java.util.Comparator;
import java.util.Hashtable;

/**
 * Class for the amino acids.
 * 
 * @author ng
 */
public class AminoAcid {
	private char shortdescr = ' ';
	private String longdescr = null;

	// **
	// The known acids as constants (static members).
	// **

	public static AminoAcid Alanine = null;
	public static AminoAcid Cysteine = null;
	public static AminoAcid Aspartic_acid = null;
	public static AminoAcid Glutamic_acid = null;
	public static AminoAcid Pheylalanine = null;
	public static AminoAcid Glycinne = null;
	public static AminoAcid Histidine = null;
	public static AminoAcid Isoleucine = null;
	public static AminoAcid Lysine = null;
	public static AminoAcid Leucine = null;
	public static AminoAcid Methionine = null;
	public static AminoAcid Asparagine = null;
	public static AminoAcid Proline = null;
	public static AminoAcid Glutamine = null;
	public static AminoAcid Arginine = null;
	public static AminoAcid Serine = null;
	public static AminoAcid Threonine = null;
	public static AminoAcid Valine = null;
	public static AminoAcid Tryptophan = null;
	public static AminoAcid Tyrosine = null;
	// Termination acid
	public static AminoAcid Term = null;

	/**
	 * Mapping of short description (1 character / index of that 1 character) to
	 * acid.
	 */
	private static AminoAcid[] short2acid = null;
	/**
	 * Mapping of long description (3 Letters) to acid
	 */
	private static Hashtable<String, AminoAcid> long2acid = null;

	/**
	 * The comparator for two amino acid. The comparison is executed by ordering
	 * the long description.
	 */
	class AcidComparator implements Comparator<AminoAcid> {
		public AcidComparator() {
			super();
		}

		public int compare(AminoAcid a1, AminoAcid a2) {
			return a1.getLongDescr().compareTo(a2.getLongDescr());
		}
	}

	private AminoAcid(char shortdescr, String longdescr) {
		super();
		this.shortdescr = shortdescr;
		this.longdescr = longdescr;
		short2acid[getIndex(shortdescr)] = this;
		long2acid.put(longdescr, this);
	}

	public boolean equals(Object o) {
		if (o == null || !(o instanceof AminoAcid)) {
			return false;
		}
		AminoAcid acid = (AminoAcid) o;
		return this.shortdescr == acid.shortdescr;
	}

	public static AminoAcid get(char shortdescr) {
		return short2acid[getIndex(shortdescr)];
	}

	public static AminoAcid get(String longdescr) {
		return long2acid.get(longdescr);
	}

	/**
	 * Returns all acids sorted by long description.
	 */
	public static AminoAcid[] getSortedAcids() {
		String[] keys = long2acid.keySet().toArray(new String[long2acid.size()]);
		Arrays.sort(keys);
		AminoAcid[] acids = new AminoAcid[keys.length];
		for (int i = 0; i < keys.length; i++) {
			acids[i] = long2acid.get(keys[i]);
		}
		return acids;
	}

	public char getShortDescr() {
		return shortdescr;
	}

	public String getLongDescr() {
		return longdescr;
	}

	public int hashCode() {
		return longdescr.hashCode();
	}

	public String toString() {
		return longdescr;
	}

	// **
	// Private
	// **

	private static int getIndex(char shortdescr) {
		return shortdescr;
	}

	static {
		short2acid = new AminoAcid[256];
		long2acid = new Hashtable<String, AminoAcid>();

		Alanine = new AminoAcid('A', "Ala");
		Cysteine = new AminoAcid('C', "Cys");
		Aspartic_acid = new AminoAcid('D', "Asp");
		Glutamic_acid = new AminoAcid('E', "Glu");
		Pheylalanine = new AminoAcid('F', "Phe");
		Glycinne = new AminoAcid('G', "Gly");
		Histidine = new AminoAcid('H', "His");
		Isoleucine = new AminoAcid('I', "Ile");
		Lysine = new AminoAcid('K', "Lys");
		Leucine = new AminoAcid('L', "Leu");
		Methionine = new AminoAcid('M', "Met");
		Asparagine = new AminoAcid('N', "Asn");
		Proline = new AminoAcid('P', "Pro");
		Glutamine = new AminoAcid('Q', "Gln");
		Arginine = new AminoAcid('R', "Arg");
		Serine = new AminoAcid('S', "Ser");
		Threonine = new AminoAcid('T', "Thr");
		Valine = new AminoAcid('V', "Val");
		Tryptophan = new AminoAcid('W', "Trp");
		Tyrosine = new AminoAcid('Y', "Tyr");

		Term = new AminoAcid(' ', "Ter");
	}
}
