package de.bioinf.base;

import de.bioinf.utils.BioinfException;
import de.bioinf.utils.LineReader;
import de.bioinf.utils.SequenceSelector;

/**
 * A sequence in a fasta file. A sequence has a header line (Starting with ">") and a set of lines
 * as data.
 * 
 * @author ng
 */
public class FastaSeq {
	/**
	 * The header (starting with ">")
	 */
	public String header = null;

	/**
	 * The sequence
	 */
	public String seq = null;

	public FastaSeq(String header, String seq) {
		this.header = header;
		this.seq = seq;
	}

	/**
	 * Reads the next sequence of a fasta file. Returns null, if no sequence is available any more.
	 */
	public static FastaSeq nextSeq(LineReader reader, SequenceSelector selector) throws BioinfException {
		SequenceHeader header = null;
		StringBuffer seq = new StringBuffer();
		String line = null;

		while ((line = reader.readLine()) != null) {
			line = line.trim();
			if (line.startsWith(";")) {
				continue;
			}
			if (line.startsWith(">")) {
				if (header != null) {
					reader.putBack();
					return new FastaSeq(header.toString(), seq.toString());
				}
				header = selector.isOk(line) ? new SequenceHeader(line) : null;
			} else {
				if (header != null) {
					seq.append(line);
				}
			}
		}

		return header != null ? new FastaSeq(header.toString(), seq.toString()) : null;
	}
}
