package de.bioinf.ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import de.bioinf.utils.Source;

/**
 * A button that opens a file selector dialog where the file may be chosen. Clicking on ok in the
 * dialog will put the selected filename in this component.
 * 
 * @author ng
 */
public abstract class FileSelectorButton extends Button implements ActionListener {
	private FileDialog dialog = null;
	private Source source = null;

	public static final String NO_SELECTION = "Click to select";

	private class FileDialog extends FileSelectorDialog {
		private FileDialog(String title) {
			super(title, null);
		}

		@Override
		public void afterOk(ActionEvent e) {
			FileSelectorButton.this.setText((source = new Source(getFilename())).getLocation());
			FileSelectorButton.this.afterOk();
		}
	}

	public FileSelectorButton(String title) {
		super(NO_SELECTION);
		setHorizontalAlignment(Button.LEFT);
		dialog = new FileDialog(title);
		addActionListener(this);
	}

	public void actionPerformed(ActionEvent e) {
		dialog.setVisible(true);
	}

	/**
	 * Action that is (additionally) executed when the dialog is closed by clicking on "ok".
	 */
	public abstract void afterOk();

	/**
	 * Returns the underlaying dialog.
	 */
	public FileSelectorDialog getDialog() {
		return dialog;
	}

	/**
	 * Returns the selected file as source
	 */
	public Source getSource() {
		return source;
	}

	public void reset() {
		setText(NO_SELECTION);
		source = null;
	}
}