package de.bioinf.ui;

import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import de.bioinf.utils.Logger;
import de.bioinf.utils.ValueRange;

/**
 * A component for numeric input with comma.
 * 
 * @author ng
 */
public class FloatTextField extends TextField {
	private ValueRange range = null;

	private static final String VALID_CHARS = new String("0123456789-." + KeyEvent.VK_LEFT + KeyEvent.VK_RIGHT
			+ KeyEvent.VK_DELETE + KeyEvent.VK_BACK_SPACE);

	private class Listener implements KeyListener {
		public void keyPressed(KeyEvent ev) {
		}

		public void keyReleased(KeyEvent ev) {
		}

		public void keyTyped(KeyEvent ev) {
			char c = ev.getKeyChar();
			if (VALID_CHARS.indexOf(c) < 0) {
				ev.consume();
				Toolkit.getDefaultToolkit().beep();
			} else if (c == '.' && getText().indexOf('.') < 0) {
				ev.consume();
				Toolkit.getDefaultToolkit().beep();
			} else if (c == KeyEvent.VK_MINUS) {
				ev.consume();
				if (getText().length() > 0) {
					setText(Double.toString(-Double.parseDouble(getText())));
				}
			}
		}
	}

	public FloatTextField(double value, ValueRange range) {
		super(Double.toString(value));
		this.range = range;
		addKeyListener(new Listener());
	}

	public FloatTextField(double value, double min, double max) {
		this(value, new ValueRange(min, max));
	}

	public FloatTextField(double value) {
		this(value, Double.MIN_VALUE, Double.MAX_VALUE);
	}

	public FloatTextField(double min, double max) {
		this(min, min, max);
	}

	public double getValue() {
		return getText().length() > 0 ? Double.parseDouble(getText()) : 0;
	}

	public void setValue(double value) {
		setText(Double.toString(value));
	}

	public boolean isOk() {
		boolean ok = range.isInside(getValue());
		if (!ok) {
			Logger.info(String.format("Number must be between %f and %f!", range.getMinVal(), range.getMaxVal()));
		}
		return ok;
	}
}
