package de.bioinf.ui;

import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import de.bioinf.utils.IntValueRange;
import de.bioinf.utils.Logger;

/**
 * A swing component for numeric input.
 * 
 * @author ng
 */
public class IntegerTextField extends TextField {
	private IntValueRange range = null;

	private static final String VALID_CHARS = new String("0123456789-" + KeyEvent.VK_LEFT + KeyEvent.VK_RIGHT
			+ KeyEvent.VK_DELETE + KeyEvent.VK_BACK_SPACE);

	private class Listener implements KeyListener {
		public void keyPressed(KeyEvent ev) {
		}

		public void keyReleased(KeyEvent ev) {
		}

		public void keyTyped(KeyEvent ev) {
			char c = ev.getKeyChar();
			if (VALID_CHARS.indexOf(c) < 0) {
				ev.consume();
				Toolkit.getDefaultToolkit().beep();
			} else if (c == KeyEvent.VK_MINUS) {
				ev.consume();
				if (getText().length() > 0) {
					setText(Integer.toString(-Integer.parseInt(getText())));
				}
			}
		}
	}

	public IntegerTextField(int value, IntValueRange range) {
		super(Integer.toString(value));
		this.range = range;
		addKeyListener(new Listener());
	}

	public IntegerTextField(int value, int min, int max) {
		this(value, new IntValueRange(min, max));
	}

	public IntegerTextField(int value) {
		this(value, Integer.MIN_VALUE, Integer.MAX_VALUE);
	}

	public IntegerTextField(int min, int max) {
		this(min, min, max);
	}

	public int getValue() {
		return getText().length() > 0 ? Integer.parseInt(getText()) : 0;
	}

	public void setValue(int value) {
		setText(Integer.toString(value));
	}

	public boolean isOk() {
		boolean ok = range.isInside(getValue());
		if (!ok) {
			Logger.info(String.format("Number must be between %d and %d!", range.getMinVal(), range.getMaxVal()));
		}
		return ok;
	}
}
