package de.bioinf.ui;

import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import de.bioinf.base.NucleotidSequence;
import de.bioinf.utils.Logger;

/**
 * A swing component for nucleotid sequence input. The entered text is displayed
 * in capital letters and only valid nucleotid characters are accepted.
 * 
 * @author ng
 */
public class NucleotidSequenceTextField extends TextField {
	class Listener implements KeyListener {
		public void keyPressed(KeyEvent ev) {
		}

		public void keyReleased(KeyEvent ev) {
		}

		public void keyTyped(KeyEvent ev) {
			char c = Character.toUpperCase(ev.getKeyChar());
			if ("ACGT".indexOf(c) < 0) {
				ev.consume();
				Toolkit.getDefaultToolkit().beep();
			} else {
				ev.setKeyChar(c);
			}
		}
	}

	public NucleotidSequenceTextField(NucleotidSequence nseq) {
		super();
		this.setText(nseq.toString());
		addKeyListener(new Listener());
	}

	public NucleotidSequenceTextField() {
		this(new NucleotidSequence(""));
	}

	public NucleotidSequence getValue() {
		return new NucleotidSequence(getText());
	}

	public boolean isOk() {
		boolean ok = getText().length() > 0;
		if (!ok) {
			Logger.info("Length must be > 0!");
		}
		return ok;
	}
}
