package de.bioinf.ui;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import de.bioinf.utils.Logger;

// TODO Find a better solution for savedPart and part behaviour used as an InputComponent
/**
 * A panel made for partitioning values.
 * 
 * @see Partition
 * 
 * @author ng
 */
public class PartitionPanel<T> extends JPanel implements InputComponent {
	private Partition<T> part = null;
	private Partition<T> savedPart = null;

	private ValueSection g_value = null;
	private PartitionSection g_part = null;
	private PartValueSection g_pvalue = null;
	private JTextField g_pname = null;

	private abstract class Section extends SectionPanel implements ActionListener {
		public Section(String title) {
			super(title);
		}

		protected Button getButton(String title) {
			Button b = new Button(title);
			b.addActionListener(this);
			return b;
		}
	}

	private class PartitionSection extends Section implements ListSelectionListener, ActionListener {
		JList g_list = null;
		Button g_add = null;
		Button g_delete = null;
		Button g_rename = null;

		public PartitionSection(String title) {
			super(title);

			JPanel buttons = new JPanel(new GridLayout(1, 3));
			buttons.add(g_add = getButton("Add"));
			buttons.add(g_delete = getButton("Del"));
			buttons.add(g_rename = getButton("Ren"));

			JPanel options = new JPanel(new BorderLayout());
			options.add(g_pname = new TextField(), BorderLayout.NORTH);
			options.add(buttons, BorderLayout.SOUTH);

			JPanel p = new JPanel(new BorderLayout());
			p.add(new JScrollPane(g_list = new JList()), BorderLayout.CENTER);
			p.add(options, BorderLayout.SOUTH);

			addSectionComponent(p);

			g_list.addListSelectionListener(this);
		}

		public void init() {
			g_list.setListData(part.getPartitions());
		}

		public String getSingleSelection() {
			Object[] values = g_list.getSelectedValues();
			return values.length == 1 ? (String) values[0] : null;
		}

		@Override
		public void valueChanged(ListSelectionEvent e) {
			if (e.getValueIsAdjusting()) {
				String value = getSingleSelection();
				if (value != null) {
					g_pvalue.init(value);
					g_pname.setText(value);
				} else {
					g_pvalue.clear();
					g_pname.setText("");
				}
			}
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == g_add) {
				String pname = g_pname.getText().trim();
				part.add(pname);
				PartitionPanel.this.init(pname);
				g_list.setSelectedValue(pname, true);
			} else if (e.getSource() == g_delete) {
				Object[] pnames = g_list.getSelectedValues();
				for (Object pname : pnames) {
					part.deletePartition((String) pname);
				}
				PartitionPanel.this.init(part);
			} else if (e.getSource() == g_rename) {
				if (Logger.info(getSingleSelection() != null, "Please select the element to be renamed!")) {
					String to = g_pname.getText().trim();
					if (to.length() > 0) {
						String from = getSingleSelection();
						if (from != null) {
							part.renamePartition(from, to);
							init();
							g_list.setSelectedValue(to, true);
						}
					}
				}
			}
		}
	}

	private class PartValueSection extends Section {
		JList g_list = null;

		public PartValueSection(String title) {
			super(title);

			JPanel options = new JPanel(new BorderLayout());
			options.add(getButton(">"), BorderLayout.SOUTH);

			JPanel p = new JPanel(new BorderLayout());
			p.add(new JScrollPane(g_list = new JList()), BorderLayout.CENTER);
			p.add(options, BorderLayout.SOUTH);

			addSectionComponent(p);
		}

		public void init(String pname) {
			g_list.setListData(part.getValues(pname));
		}

		public void clear() {
			g_list.setListData(new Object[0]);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			Object[] values = g_list.getSelectedValues();
			if (values.length > 0) {
				for (Object o : values) {
					part.unlink((T) o);
				}
				init(g_part.getSingleSelection());
				g_value.init();
			}
		}
	}

	private class ValueSection extends Section {
		JList g_list = null;

		public ValueSection(String title) {
			super(title);

			JPanel options = new JPanel(new BorderLayout());
			options.add(getButton("<"), BorderLayout.SOUTH);

			JPanel p = new JPanel(new BorderLayout());
			p.add(new JScrollPane(g_list = new JList()), BorderLayout.CENTER);
			p.add(options, BorderLayout.SOUTH);

			addSectionComponent(p);
		}

		public void init() {
			g_list.setListData(part.getFreeValues());
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			String pname = g_part.getSingleSelection();
			if (pname != null) {
				Object[] values = g_list.getSelectedValues();
				if (values.length > 0) {
					for (Object o : values) {
						part.link((T) o, pname);
					}
					init();
					g_pvalue.init(pname);
				}
			}
		}
	}

	/**
	 * Create a partition panel.
	 * 
	 * @param part - the partition
	 * @param pheader - the header for the partition section
	 * @param pvheader - the header for the partition values section
	 * @param vheader - the header for the (free) values section
	 */
	public PartitionPanel(Partition<T> part, String pheader, String pvheader, String vheader) {
		super(new GridLayout(1, 3, 3, 1));

		add(g_part = new PartitionSection(pheader));
		add(g_pvalue = new PartValueSection(pvheader));
		add(g_value = new ValueSection(vheader));

		init(part);
	}

	public void addValues(T[] values) {
		// A dirty trick to add values to both lists.
		part.addValues(values);
		savedPart.addValues(values);
		g_value.init();
	}

	public Partition<T> getPartition() {
		return part;
	}

	/**
	 * Choosing an already existing partition name.
	 */
	public void init(String pname) {
		if (part.isPartition(pname)) {
			g_value.init();
			g_part.init();
			g_pvalue.init(pname);
		}
	}

	/**
	 * Initialize with a new partition
	 */
	public void init(Partition<T> part) {
		this.savedPart = part;
		this.part = part.copy();
		g_value.init();
		g_part.init();
		g_pvalue.clear();
	}

	@Override
	public boolean isOk() {
		return true;
	}

	@Override
	public void reset() {
		init(savedPart);
	}

	@Override
	public void saveState() {
		savedPart = part.copy();
	}
}
