package de.bioinf.ui.graph;

import de.bioinf.utils.ValueRange;

/**
 * The base class for elements to be added to a HorizBarPanel.
 * 
 * @author ng
 */
public abstract class HorizBarElem extends GraphElem {
	/**
	 * The base panel (upon which this element will be displayed).
	 */
	protected HorizBarPanel panel = null;
	/**
	 * The range of the x values.
	 */
	private ValueRange xrange = null;
	/**
	 * The height of this line (in pixels)
	 */
	private int height = 0;
	/**
	 * The y position. This is stored in this element to avoid calculating in LinePanel using getY(LineElem).
	 */
	private int y = 0;

	/**
	 * Creates a line element, but does NOT add it to the underlaying panel.
	 * 
	 * @param xrange - the value range of x
	 * @param height - the height in pixel
	 * @param id - information that are displayed when clicked.
	 */
	public HorizBarElem(HorizBarPanel panel, ValueRange xrange, int height, String id) {
		super(id);
		this.panel = panel;
		this.xrange = xrange;
		this.height = height;
	}

	@Override
	public void adjustRanges(ValueRange xrange, ValueRange yrange) {
		xrange.add(this.xrange);
	}

	/**
	 * Limits the visibility to elements where the y value is inside the range of the display area.
	 */
	@Override
	public boolean isVisible() {
		return y + height >= 0 && y <= panel.getHeight();
	}

	public final int getHeight() {
		return height;
	}

	public final double getMinXVal() {
		return xrange.getMinVal();
	}

	public final double getMaxXVal() {
		return xrange.getMaxVal();
	}

	/**
	 * Returns the minimal (left) x position
	 */
	public final int getMinX() {
		return panel.getMinX(this);
	}

	/**
	 * Returns the maximal (right) x position
	 */
	public final int getMaxX() {
		return panel.getMaxX(this);
	}

	public final int getY() {
		return y;
	}

	/**
	 * Set the height of the bar
	 */
	public void setHeight(int height) {
		this.height = height;
		panel.setChanged();
	}

	/**
	 * Sets the x coordinate range
	 */
	public void setXRange(ValueRange range) {
		xrange = range;
		panel.setChanged();
	}

	/**
	 * Set the y position.
	 */
	protected void setY(int y) {
		this.y = y;
	}
}
