package de.bioinf.utils;

import java.util.Stack;

/**
 * The main exception class
 * 
 * @author ng
 */
public class BioinfException extends Exception {
	private Stack<String> messages = new Stack<String>();

	/**
	 * An empty exception
	 */
	public BioinfException() {
		super();
	}

	/**
	 * Creates an exception with the given message
	 */
	public BioinfException(String message) {
		this();
		addMessage(message);
	}

	/**
	 * Creates an exception for the given Throwable
	 */
	public BioinfException(Throwable t) {
		this(t.getMessage() != null && t.getMessage().trim().length() == 0 ? t.getMessage() : t.toString());
	}

	/**
	 * Adds a message to the already existing messages.
	 */
	public BioinfException addMessage(String message) {
		messages.push(message);
		return this;
	}

	/**
	 * Converts any Throwable to a BioinfException
	 */
	public static BioinfException convert(Throwable t) {
		return (t instanceof BioinfException) ? (BioinfException) t : new BioinfException(t);
	}

	/**
	 * Fires an empty exception if the condition is false
	 */
	public static void fire(boolean cond) throws BioinfException {
		if (!cond) {
			throw new BioinfException();
		}
	}

	/**
	 * Fire an exception with the message if the condition is false.
	 */
	public static void fire(boolean cond, String message) throws BioinfException {
		if (!cond) {
			throw new BioinfException(message);
		}
	}

	/**
	 * Fire an exception with objectname and message if the condition is false.
	 */
	public static void fire(boolean cond, Object o, String message) throws BioinfException {
		if (!cond) {
			throw new BioinfException("[" + Utils.getName(o) + "] " + message);
		}
	}

	/**
	 * Returns all messages (even the added ones) - but in he order last added to first added.
	 */
	public String getMessage() {
		StringBuffer buf = new StringBuffer();
		if (messages.size() > 0) {
			buf.append(messages.pop());
		}
		while (messages.size() > 0) {
			buf.append(String.format(" [%s]", messages.pop()));
		}
		return buf.toString();
	}

	public String toString() {
		return getMessage();
	}
}
