package de.bioinf.utils;

import java.io.BufferedWriter;
import java.io.FileWriter;

public class LineWriter {
	private String filename = null;
	private BufferedWriter writer = null;
	private int lineno = 0;

	public LineWriter(String filename) throws BioinfException {
		super();
		try {
			writer = new BufferedWriter(new FileWriter(filename));
			this.filename = filename;
		} catch (Exception e) {
			throw BioinfException.convert(e).addMessage(String.format("Error opening file %s!", filename));
		}
	}

	public void writeLine(String line) throws BioinfException {
		try {
			writer.write(line);
			writer.newLine();
			lineno++;
		} catch (Exception e) {
			throw new BioinfException(String.format("Error writing line %d to file %s!", lineno, filename));
		}
	}

	public void close() throws BioinfException {
		try {
			writer.close();
		} catch (Exception e) {
			throw BioinfException.convert(e).addMessage(String.format("Error closing file %s!", filename));
		}
	}

	/**
	 * Returns the last written line number. Line number 0 means no line is read so far.
	 */
	public int getLineNumber() {
		return lineno;
	}
}
