package de.bioinf.utils;

import java.util.Arrays;
import java.util.HashMap;

/**
 * This class is used to manage parameter (key, value) entries. Even null values are valid! It is possible to add
 * listeners for changes.
 * 
 * @author ng
 */
public abstract class Params<K, V> {
	protected HashMap<K, V> params = new HashMap<K, V>();
	protected boolean dirty = false;

	public Params() {
		super();
	}

	/**
	 * Removes all entries from the parameters.
	 */
	public void clear() {
		params.clear();
		dirty = true;
	}

	/**
	 * There is not other way to create an array of unknown type than in an implementing class.
	 */
	public abstract K[] getKeys();

	public K[] getSortedKeys() {
		K[] keys = getKeys();
		Arrays.sort(keys);
		return keys;
	}

	public V getValue(K key) {
		return params.get(key);
	}

	public boolean isKey(K key) {
		return params.containsKey(key);
	}

	public void put(K key, V value) {
		params.put(key, value);
		dirty = true;
	}

	public void remove(K key) {
		params.remove(key);
		dirty = true;
	}

	public String toString() {
		StringBuffer buf = new StringBuffer("[");
		K[] keys = getSortedKeys();
		for (int i = 0; i < keys.length; i++) {
			if (i > 0) {
				buf.append(",");
			}
			buf.append(keys[i]).append("=").append(params.get(keys[i]));
		}
		buf.append("]");
		return buf.toString();
	}
}
