package de.bioinf.utils;

import java.io.IOException;

import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

import de.bioinf.ui.Borders;

public class Www {
	/**
	 * The base url part of our institute
	 */
	public static final String URL_BASE = "http://www.bioinformatics.uni-muenster.de";

	static class EditorPane extends JEditorPane implements HyperlinkListener {
		public EditorPane(String url) throws IOException {
			super(url);
			addHyperlinkListener(this);
		}

		public void hyperlinkUpdate(HyperlinkEvent ev) {
			if (ev.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
				try {
					setPage(ev.getURL());
				} catch (Exception e) {
					Logger.error(String.format("Cannot display page %s", ev.getURL()), e);
				}
			}
		}
	}

	/**
	 * Returns a component where the page is related to a property set in an jnlp file (see the resources section). If
	 * the property is not set, the page "/errors/notfound.html" is returned.
	 */
	public static JComponent getPageByProperty(String name) {
		String prop = System.getProperty(name);
		return getPage(prop == null ? "/errors/notfound.html" : prop);
	}

	/**
	 * Reads the page and returns a component. If the page is specified only like in a filesystem like
	 * "/dir1/dir2/page.html?params", it is assumed that is relays on the bioinformatics server.
	 */
	public static JComponent getPage(String page) {
		try {
			if (!page.matches("http:")) {
				page = String.format("%s%s?layout=simple", URL_BASE, page);
			}
			EditorPane pane = new EditorPane(page);
			pane.setEditable(false);
			JScrollPane spane = new JScrollPane(pane);
			spane.setBorder(new Borders(0));
			return spane;
		} catch (Exception e) {
			Logger.error(String.format("Cannot create page %s", page), e);
		}
		return null;
	}

}
